# -*- coding: utf-8 -*-
"""
Created on Sun Feb 28 17:03:48 2021

@author: Daniel
"""

import pytest
from accim.sim import accim_Main

@pytest.fixture()
def IDFobject():
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    idf0 = IDF('TestModel_SingleZone.idf')

    programlist = ([program.Name for program in idf0.idfobjects['EnergyManagementSystem:Program']])
    z = accim_Main.accimobj_SingleZone_Ep94(filename_temp='TestModel_SingleZone', verboseMode=False)
    return z

def test_addEMSProgramsBase(IDFobject):

    from eppy.modeleditor import IDF

    IDFobject.addEMSProgramsBase(verboseMode=False)
    IDFobject.saveaccim()

    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    programlist = ([program.Name for program in idf1.idfobjects['EnergyManagementSystem:Program']])
    assert ('SetComfTemp' in programlist) == True

    SetComfTemp = ([program
                    for program
                    in idf1.idfobjects['EnergyManagementSystem:Program']
                    if program.Name == 'SetComfTemp'])

    assert SetComfTemp[0].Program_Line_1 == 'if AdapStand == 1'
    assert SetComfTemp[0].Program_Line_2 == 'set ComfTemp = RMOT*0.33+18.8'
    assert SetComfTemp[0].Program_Line_3 == 'elseif AdapStand == 2'
    assert SetComfTemp[0].Program_Line_4 == 'set ComfTemp = PMOT*0.31+17.8'
    assert SetComfTemp[0].Program_Line_5 == 'endif'

        # for zonename in self.zonenames:

# y = tests_SingleZone()
# y.test_addEMSProgramsBase()



"""
Multiprocessing runs.

using generators instead of a list
when you are running a 100 files you have to use generators
original script: https://eppy.readthedocs.io/en/latest/runningeplus.html
slightly modified so that takes multiple EPWs located in the local folder
"""

import os
from eppy.modeleditor import IDF
from eppy.runner.run_functions import runIDFs


def make_eplaunch_options(idf, epw):
    """Make options for run, so that it runs like EPLaunch on Windows."""
    idfversion = idf.idfobjects['version'][0].Version_Identifier.split('.')
    idfversion.extend([0] * (3 - len(idfversion)))
    idfversionstr = '-'.join([str(item) for item in idfversion])
    epw = epw.split('.')[0]
    fname = idf.idfname + '_' + epw
    options = {
        'ep_version': idfversionstr,  # runIDFs needs the version number
        'output_prefix': os.path.basename(fname).split('.idf')[0]+'['+epw,
        'output_suffix': 'C',
        'output_directory': os.path.dirname(fname),
        'readvars': True,
        'expandobjects': True
        }
    return options


def runEp94(runOnlyAccim=None, confirmRun=None, num_CPUs: int = 2):
    """
    Run simulations in Energy Plus 9.4.0.

    :param runOnlyAccim: Default is None. Enter True to run only ACCIM output IDFs, or False to run all IDFs.
    :param confirmRun: Default is None. Enter True to run all simulations regardless the no. of them,
    or False to shut down all runs.
    :return:
    """
    iddfile = "C:/EnergyPlusV9-4-0/Energy+.idd"
    IDF.setiddname(iddfile)

    if runOnlyAccim is None:
        runOnlyAccim = input('Do you want to run only ACCIM output IDFs? [y or n]: ')
        if runOnlyAccim.lower() == 'y' or runOnlyAccim.lower() == '':
            idfnames = [x for x in os.listdir() if x.endswith('.idf') and '_pymod' in x]
        else:
            idfnames = [x for x in os.listdir() if x.endswith('.idf')]
    elif runOnlyAccim:
        idfnames = [x for x in os.listdir() if x.endswith('.idf') and '_pymod' in x]
    else:
        idfnames = [x for x in os.listdir() if x.endswith('.idf')]

    epwnames = [x for x in os.listdir() if x.endswith('.epw')]
    epwnames_run = [x.split('.epw')[0] for x in os.listdir() if x.endswith('.epw')]

    # if IDFfilesPath is None:
    # else:
    #     if runOnlyAccim.lower() == 'y' or runOnlyAccim.lower() == '':
    #         idfnames = [x for x in os.listdir(IDFfilesPath) if x.endswith('.idf') and '_pymod' in x]
    #     else:
    #         idfnames = [x for x in os.listdir(IDFfilesPath) if x.endswith('.idf')]
    #
    # if EPWfilesPath is None:
    #     epwnames = [x for x in os.listdir() if x.endswith('.epw')]
    #     epwnames_run = [x.split('.epw')[0] for x in os.listdir() if x.endswith('.epw')]
    # else:
    #     epwnames = [x for x in os.listdir(EPWfilesPath) if x.endswith('.epw')]
    #     epwnames_run = [x.split('.epw')[0] for x in os.listdir(EPWfilesPath) if x.endswith('.epw')]

    print(f'The IDFs we are going to run are:')
    print(*idfnames, sep="\n")
    print(f' and the No. of IDFs is going to be {len(idfnames)}')
    print(f'The EPWs we are going to run are:')
    print(*epwnames, sep="\n")
    print(f' and the No. of EPWs is going to be {len(epwnames)}')

    print('Therefore, the simulations are going to be:')
    idfs = []
    for i in idfnames:
        for j in epwnames:
            tempidf = IDF(i, j)
            print(i.split('.idf')[0] + '[' + j)
            idfs.append(tempidf)
    print(f' and the No. of simulations is going to be {len(idfs)}')
    # print(idfs)

    runs = []
    for i in idfs:
        for j in epwnames_run:
            temprun = (i, make_eplaunch_options(i, j))
            if i.epw == j+'.epw':
                # print(temprun)
                runs.append(temprun)
            else:
                continue

    if confirmRun is None:
        confirmRun = input(
            f'The number of simulations is going to be {len(runs)}. Do you still want to proceed? [y or n]: ')
        if confirmRun == 'y':
            runIDFs(runs, num_CPUs)
        else:
            print('Run has been shut down')
    elif confirmRun:
        runIDFs(runs, num_CPUs)
    else:
        print('Run has been shut down')


def removefiles():
    """
    Delete all files except '.py', '.idf', '.epw' and '.csv'.

    'Table.csv', 'Zsz.csv' files are deleted as well.
    """
    extensions = ('.py', '.idf', '.epw', '.csv')
    csvextensions = ('Table.csv', 'Zsz.csv')
    deletelist = ([file for file in os.listdir() if not file.endswith(extensions)])
    for file in deletelist:
        os.remove(file)
    csvlist = ([file for file in os.listdir() if file.endswith(csvextensions)])
    for file in csvlist:
        os.remove(file)



import pytest
from accim.sim import accim_Main

@pytest.fixture()
def IDFobject():
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimInstance(
        filename_temp='TestModel_SingleZone',
        ScriptType='sz',
        EnergyPlus_version='ep94',
        verboseMode=False)
    return z

def test_setComfFieldsPeople(IDFobject):
    from eppy.modeleditor import IDF
    
    IDFobject.setComfFieldsPeople(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    peoplelist = ([people for people in idf1.idfobjects['PEOPLE']])
    for i in range(len(peoplelist)):
        assert peoplelist[i].Name == peoplelist[i].Name
        assert peoplelist[i].Zone_or_ZoneList_Name == peoplelist[i].Zone_or_ZoneList_Name
        assert peoplelist[i].Number_of_People_Schedule_Name == peoplelist[i].Number_of_People_Schedule_Name
        assert peoplelist[i].Number_of_People_Calculation_Method == peoplelist[i].Number_of_People_Calculation_Method
        assert peoplelist[i].Number_of_People == peoplelist[i].Number_of_People
        assert peoplelist[i].People_per_Zone_Floor_Area == peoplelist[i].People_per_Zone_Floor_Area
        assert peoplelist[i].Zone_Floor_Area_per_Person == peoplelist[i].Zone_Floor_Area_per_Person
        assert peoplelist[i].Fraction_Radiant == peoplelist[i].Fraction_Radiant
        assert peoplelist[i].Sensible_Heat_Fraction == peoplelist[i].Sensible_Heat_Fraction
        assert peoplelist[i].Activity_Level_Schedule_Name == peoplelist[i].Activity_Level_Schedule_Name
        assert peoplelist[i].Carbon_Dioxide_Generation_Rate == peoplelist[i].Carbon_Dioxide_Generation_Rate
        assert peoplelist[i].Enable_ASHRAE_55_Comfort_Warnings == peoplelist[i].Enable_ASHRAE_55_Comfort_Warnings
        assert peoplelist[i].Mean_Radiant_Temperature_Calculation_Type == peoplelist[i].Mean_Radiant_Temperature_Calculation_Type
        assert peoplelist[i].Surface_NameAngle_Factor_List_Name == peoplelist[i].Surface_NameAngle_Factor_List_Name
        assert peoplelist[i].Work_Efficiency_Schedule_Name == peoplelist[i].Work_Efficiency_Schedule_Name
        assert peoplelist[i].Clothing_Insulation_Calculation_Method == peoplelist[i].Clothing_Insulation_Calculation_Method
        assert peoplelist[i].Clothing_Insulation_Calculation_Method_Schedule_Name == peoplelist[i].Clothing_Insulation_Calculation_Method_Schedule_Name
        assert peoplelist[i].Clothing_Insulation_Schedule_Name == peoplelist[i].Clothing_Insulation_Schedule_Name
        assert peoplelist[i].Air_Velocity_Schedule_Name == peoplelist[i].Air_Velocity_Schedule_Name
        assert peoplelist[i].Thermal_Comfort_Model_1_Type == 'AdaptiveASH55'
        assert peoplelist[i].Thermal_Comfort_Model_2_Type == 'AdaptiveCEN15251'
        assert peoplelist[i].Thermal_Comfort_Model_3_Type == ''
        assert peoplelist[i].Thermal_Comfort_Model_4_Type == ''
        assert peoplelist[i].Thermal_Comfort_Model_5_Type == ''


def test_addOpTempTherm(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addOpTempTherm(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    zonenames_orig = ([zone.Name for zone in idf1.idfobjects['ZONE']])
    for zonename_orig in zonenames_orig:
        ZoneControl = ([x
                        for x
                        in idf1.idfobjects['ZoneControl:Thermostat:OperativeTemperature']
                        if x.Thermostat_Name == zonename_orig+" Thermostat"])
        assert ZoneControl[0].Thermostat_Name == zonename_orig + " Thermostat"
        assert ZoneControl[0].Radiative_Fraction_Input_Mode == "Scheduled"
        assert ZoneControl[0].Fixed_Radiative_Fraction == ''
        assert ZoneControl[0].Radiative_Fraction_Schedule_Name == 'TypOperativeTempControlSch'


def test_addBaseSchedules(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addBaseSchedules(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    sch = ([x
            for x
            in idf1.idfobjects['Schedule:Compact']
            if x.Name == 'On'])
    assert sch[0].Name == "On"
    assert sch[0].Schedule_Type_Limits_Name == "Any Number"
    assert sch[0].Field_1 == 'Through: 12/31'
    assert sch[0].Field_2 == 'For: AllDays'
    assert sch[0].Field_3 == 'Until: 24:00'
    assert sch[0].Field_4 == '1'


def test_setAvailSchOn(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.setAvailSchOn(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    for schedule in [i for i in idf1.idfobjects['ZoneHVAC:IdealLoadsAirSystem']]:
        assert schedule.Heating_Availability_Schedule_Name == 'On'
        assert schedule.Cooling_Availability_Schedule_Name == 'On'



"""Example of script usage with arguments."""

from accim.sim import accis

scriptTypeList = ['mz', 'sz']
outputsList = ['simplified', 'standard', 'timestep']
EPlist = ['ep94']

for i in scriptTypeList:
    for j in outputsList:
        for k in EPlist:
            accis.addAccis_v03(i,
                               j,
                               k,
                               [1], [3], [2], [0], [0], [0], [50], [50], 0.1, 0.1, 0.1,
                               j)



import pytest
from accim.sim import accim_Main

@pytest.fixture()
def IDFobject():
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-1-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimInstance(
        filename_temp='TestModel_MultipleZone',
        ScriptType='mz',
        EnergyPlus_version='ep91',
        verboseMode=False)
    return z


def test_addDetHVACobjEp91(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addDetHVACobjEp91(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_MultipleZone_pymod.idf')
    
    zonenames_orig = ([zone.Name for zone in idf1.idfobjects['ZONE']])

    for zonename in zonenames_orig:
        VRFobj = ([x
                   for x
                   in idf1.idfobjects['AirConditioner:VariableRefrigerantFlow']
                   if x.Heat_Pump_Name == 'VRF Outdoor Unit_' + zonename])
        assert VRFobj[0].Heat_Pump_Name == 'VRF Outdoor Unit_' + zonename
        assert VRFobj[0].Availability_Schedule_Name == 'On 24/7'
        assert VRFobj[0].Gross_Rated_Total_Cooling_Capacity == 'autosize'
        assert VRFobj[0].Gross_Rated_Cooling_COP == 2
        assert VRFobj[0].Minimum_Outdoor_Temperature_in_Cooling_Mode == -6
        assert VRFobj[0].Maximum_Outdoor_Temperature_in_Cooling_Mode == 43
        assert VRFobj[0].Cooling_Capacity_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name == 'VRFCoolCapFT'
        assert VRFobj[0].Cooling_Capacity_Ratio_Boundary_Curve_Name == 'VRFCoolCapFTBoundary'
        assert VRFobj[0].Cooling_Capacity_Ratio_Modifier_Function_of_High_Temperature_Curve_Name == 'VRFCoolCapFTHi'
        assert VRFobj[0].Cooling_Energy_Input_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name == 'VRFCoolEIRFT'
        assert VRFobj[0].Cooling_Energy_Input_Ratio_Boundary_Curve_Name == 'VRFCoolEIRFTBoundary'
        assert VRFobj[0].Cooling_Energy_Input_Ratio_Modifier_Function_of_High_Temperature_Curve_Name == 'VRFCoolEIRFTHi'
        assert VRFobj[0].Cooling_Energy_Input_Ratio_Modifier_Function_of_Low_PartLoad_Ratio_Curve_Name == 'CoolingEIRLowPLR'
        assert VRFobj[0].Cooling_Energy_Input_Ratio_Modifier_Function_of_High_PartLoad_Ratio_Curve_Name == 'CoolingEIRHiPLR'
        assert VRFobj[0].Cooling_Combination_Ratio_Correction_Factor_Curve_Name == 'CoolingCombRatio'
        assert VRFobj[0].Cooling_PartLoad_Fraction_Correlation_Curve_Name == 'VRFCPLFFPLR'
        assert VRFobj[0].Gross_Rated_Heating_Capacity == 'autosize'
        assert VRFobj[0].Rated_Heating_Capacity_Sizing_Ratio == 1
        assert VRFobj[0].Gross_Rated_Heating_COP == 2.1
        assert VRFobj[0].Minimum_Outdoor_Temperature_in_Heating_Mode == -20
        assert VRFobj[0].Maximum_Outdoor_Temperature_in_Heating_Mode == 40
        assert VRFobj[0].Heating_Capacity_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name == 'VRFHeatCapFT'
        assert VRFobj[0].Heating_Capacity_Ratio_Boundary_Curve_Name == 'VRFHeatCapFTBoundary'
        assert VRFobj[0].Heating_Capacity_Ratio_Modifier_Function_of_High_Temperature_Curve_Name == 'VRFHeatCapFTHi'
        assert VRFobj[0].Heating_Energy_Input_Ratio_Modifier_Function_of_Low_Temperature_Curve_Name == 'VRFHeatEIRFT'
        assert VRFobj[0].Heating_Energy_Input_Ratio_Boundary_Curve_Name == 'VRFHeatEIRFTBoundary'
        assert VRFobj[0].Heating_Energy_Input_Ratio_Modifier_Function_of_High_Temperature_Curve_Name == 'VRFHeatEIRFTHi'
        assert VRFobj[0].Heating_Performance_Curve_Outdoor_Temperature_Type == 'WetBulbTemperature'
        assert VRFobj[0].Heating_Energy_Input_Ratio_Modifier_Function_of_Low_PartLoad_Ratio_Curve_Name == 'HeatingEIRLowPLR'
        assert VRFobj[0].Heating_Energy_Input_Ratio_Modifier_Function_of_High_PartLoad_Ratio_Curve_Name == 'HeatingEIRHiPLR'
        assert VRFobj[0].Heating_Combination_Ratio_Correction_Factor_Curve_Name == 'HeatingCombRatio'
        assert VRFobj[0].Heating_PartLoad_Fraction_Correlation_Curve_Name == 'VRFCPLFFPLR'
        assert VRFobj[0].Minimum_Heat_Pump_PartLoad_Ratio == 0.2
        assert VRFobj[0].Zone_Name_for_Master_Thermostat_Location == ''
        assert VRFobj[0].Master_Thermostat_Priority_Control_Type == 'LoadPriority'
        assert VRFobj[0].Thermostat_Priority_Schedule_Name == ''
        assert VRFobj[0].Zone_Terminal_Unit_List_Name == 'VRF Outdoor Unit_' + zonename + ' Zone List'
        assert VRFobj[0].Heat_Pump_Waste_Heat_Recovery == 'Yes'
        assert VRFobj[0].Equivalent_Piping_Length_used_for_Piping_Correction_Factor_in_Cooling_Mode == 50
        assert VRFobj[0].Vertical_Height_used_for_Piping_Correction_Factor == 15
        assert VRFobj[0].Piping_Correction_Factor_for_Length_in_Cooling_Mode_Curve_Name == 'CoolingLengthCorrectionFactor'
        assert VRFobj[0].Piping_Correction_Factor_for_Height_in_Cooling_Mode_Coefficient == 0
        assert VRFobj[0].Equivalent_Piping_Length_used_for_Piping_Correction_Factor_in_Heating_Mode == 50
        assert VRFobj[0].Piping_Correction_Factor_for_Length_in_Heating_Mode_Curve_Name == 'VRF Piping Correction Factor for Length in Heating Mode'
        assert VRFobj[0].Piping_Correction_Factor_for_Height_in_Heating_Mode_Coefficient == 0
        assert VRFobj[0].Crankcase_Heater_Power_per_Compressor == 15
        assert VRFobj[0].Number_of_Compressors == 2
        assert VRFobj[0].Ratio_of_Compressor_Size_to_Total_Compressor_Capacity == 0.5
        assert VRFobj[0].Maximum_Outdoor_DryBulb_Temperature_for_Crankcase_Heater == 5
        assert VRFobj[0].Defrost_Strategy == 'Resistive'
        assert VRFobj[0].Defrost_Control == 'Timed'
        assert VRFobj[0].Defrost_Energy_Input_Ratio_Modifier_Function_of_Temperature_Curve_Name == ''
        assert VRFobj[0].Defrost_Time_Period_Fraction == 0
        assert VRFobj[0].Resistive_Defrost_Heater_Capacity == 'autosize'
        assert VRFobj[0].Maximum_Outdoor_Drybulb_Temperature_for_Defrost_Operation == 5
        assert VRFobj[0].Condenser_Type == 'AirCooled'
        assert VRFobj[0].Condenser_Inlet_Node_Name == 'VRF Outdoor Unit_' + zonename + ' Outdoor Air Node'
        assert VRFobj[0].Condenser_Outlet_Node_Name == ''
        assert VRFobj[0].Water_Condenser_Volume_Flow_Rate == 'autosize'
        assert VRFobj[0].Evaporative_Condenser_Effectiveness == 0.9
        assert VRFobj[0].Evaporative_Condenser_Air_Flow_Rate == 'autosize'
        assert VRFobj[0].Evaporative_Condenser_Pump_Rated_Power_Consumption == 'autosize'
        assert VRFobj[0].Supply_Water_Storage_Tank_Name == ''
        assert VRFobj[0].Basin_Heater_Capacity == 0
        assert VRFobj[0].Basin_Heater_Setpoint_Temperature == 2
        assert VRFobj[0].Basin_Heater_Operating_Schedule_Name == 'On 24/7'
        assert VRFobj[0].Fuel_Type == 'Electricity'
        assert VRFobj[0].Minimum_Outdoor_Temperature_in_Heat_Recovery_Mode == -10
        assert VRFobj[0].Maximum_Outdoor_Temperature_in_Heat_Recovery_Mode == 40
        assert VRFobj[0].Heat_Recovery_Cooling_Capacity_Modifier_Curve_Name == 'VRF Heat Recovery Cooling Capacity Modifier'
        assert VRFobj[0].Initial_Heat_Recovery_Cooling_Capacity_Fraction == 0.5
        assert VRFobj[0].Heat_Recovery_Cooling_Capacity_Time_Constant == 0.15
        assert VRFobj[0].Heat_Recovery_Cooling_Energy_Modifier_Curve_Name == 'VRF Heat Recovery Cooling Energy Modifier'
        assert VRFobj[0].Initial_Heat_Recovery_Cooling_Energy_Fraction == 1
        assert VRFobj[0].Heat_Recovery_Cooling_Energy_Time_Constant == 0
        assert VRFobj[0].Heat_Recovery_Heating_Capacity_Modifier_Curve_Name == 'VRF Heat Recovery Heating Capacity Modifier'
        assert VRFobj[0].Initial_Heat_Recovery_Heating_Capacity_Fraction == 1
        assert VRFobj[0].Heat_Recovery_Heating_Capacity_Time_Constant == 0.15
        assert VRFobj[0].Heat_Recovery_Heating_Energy_Modifier_Curve_Name == 'VRF Heat Recovery Heating Energy Modifier'
        assert VRFobj[0].Initial_Heat_Recovery_Heating_Energy_Fraction == 1
        assert VRFobj[0].Heat_Recovery_Heating_Energy_Time_Constant == 0

    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['OutdoorAir:NodeList']
               if i.Node_or_NodeList_Name_1 == ('VRF Outdoor Unit_'
                                                + zonename
                                                + ' Outdoor Air Node')]
        assert obj[0].Node_or_NodeList_Name_1 == ('VRF Outdoor Unit_'
                                                  + zonename
                                                  + ' Outdoor Air Node')
    
    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['ZoneTerminalUnitList']
               if i.Zone_Terminal_Unit_List_Name == ('VRF Outdoor Unit_'
                                                     + zonename
                                                     + ' Zone List')]
        assert obj[0].Zone_Terminal_Unit_List_Name == ('VRF Outdoor Unit_'
                                                  + zonename
                                                  + ' Zone List')
        assert obj[0].Zone_Terminal_Unit_Name_1 == zonename + ' VRF Indoor Unit'

    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['ZoneControl:Thermostat']
               if i.Name == zonename + ' Thermostat']
        assert obj[0].Name == zonename + ' Thermostat'
        assert obj[0].Zone_or_ZoneList_Name == zonename
        assert obj[0].Control_Type_Schedule_Name == 'Control type schedule: Always 4'
        assert obj[0].Control_1_Object_Type == 'ThermostatSetpoint:DualSetpoint'
        assert obj[0].Control_1_Name == zonename + ' Dual SP'

        for zonename in zonenames_orig:
            obj = [i
                   for i
                   in idf1.idfobjects['Sizing:Zone']
                   if i.Zone_or_ZoneList_Name == zonename]
            assert obj[0].Zone_or_ZoneList_Name == zonename
            assert obj[0].Zone_Cooling_Design_Supply_Air_Temperature_Input_Method == 'SupplyAirTemperature'
            assert obj[0].Zone_Cooling_Design_Supply_Air_Temperature == 14
            assert obj[0].Zone_Cooling_Design_Supply_Air_Temperature_Difference == 5
            assert obj[0].Zone_Heating_Design_Supply_Air_Temperature_Input_Method == 'SupplyAirTemperature'
            assert obj[0].Zone_Heating_Design_Supply_Air_Temperature == 50
            assert obj[0].Zone_Heating_Design_Supply_Air_Temperature_Difference == 15
            assert obj[0].Zone_Cooling_Design_Supply_Air_Humidity_Ratio == 0.009
            assert obj[0].Zone_Heating_Design_Supply_Air_Humidity_Ratio == 0.004
            assert obj[0].Design_Specification_Outdoor_Air_Object_Name == zonename+' Design Specification Outdoor Air Object'
            assert obj[0].Zone_Heating_Sizing_Factor == 1.25
            assert obj[0].Zone_Cooling_Sizing_Factor == 1.15
            assert obj[0].Cooling_Design_Air_Flow_Method == 'DesignDay'
            assert obj[0].Cooling_Design_Air_Flow_Rate == 0
            assert obj[0].Cooling_Minimum_Air_Flow_per_Zone_Floor_Area == 0.00076
            assert obj[0].Cooling_Minimum_Air_Flow == 0
            assert obj[0].Cooling_Minimum_Air_Flow_Fraction == 0
            assert obj[0].Heating_Design_Air_Flow_Method == 'DesignDay'
            assert obj[0].Heating_Design_Air_Flow_Rate == 0
            assert obj[0].Heating_Maximum_Air_Flow_per_Zone_Floor_Area == 0.00203
            assert obj[0].Heating_Maximum_Air_Flow == 0.14158
            assert obj[0].Heating_Maximum_Air_Flow_Fraction == 0.3
            assert obj[0].Design_Specification_Zone_Air_Distribution_Object_Name == zonename+' Design Specification Zone Air Distribution Object'
            assert obj[0].Account_for_Dedicated_Outdoor_Air_System == 'Yes'
            assert obj[0].Dedicated_Outdoor_Air_System_Control_Strategy == 'NeutralSupplyAir'
            assert obj[0].Dedicated_Outdoor_Air_Low_Setpoint_Temperature_for_Design == 'autosize'
            assert obj[0].Dedicated_Outdoor_Air_High_Setpoint_Temperature_for_Design == 'autosize'

    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['DesignSpecification:OutdoorAir']
               if i.Name == zonename + ' Design Specification Outdoor Air Object']
        assert obj[0].Name == zonename + ' Design Specification Outdoor Air Object'
        assert obj[0].Outdoor_Air_Method == 'Flow/Person'
        assert obj[0].Outdoor_Air_Flow_per_Person == 0.00944
        assert obj[0].Outdoor_Air_Flow_per_Zone_Floor_Area == 0
        assert obj[0].Outdoor_Air_Flow_per_Zone == 0
        assert obj[0].Outdoor_Air_Flow_Air_Changes_per_Hour == 0
        assert obj[0].Outdoor_Air_Schedule_Name == 'On 24/7'
    
    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['DesignSpecification:ZoneAirDistribution']
               if i.Name == zonename + ' Design Specification Zone Air Distribution Object']
        assert obj[0].Name == zonename + ' Design Specification Zone Air Distribution Object'
        assert obj[0].Zone_Air_Distribution_Effectiveness_in_Cooling_Mode == 1
        assert obj[0].Zone_Air_Distribution_Effectiveness_in_Heating_Mode == 1
        assert obj[0].Zone_Air_Distribution_Effectiveness_Schedule_Name == ''
        assert obj[0].Zone_Secondary_Recirculation_Fraction == 0
    
    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['NodeList']
               if i.Name == zonename+' Air Inlet Node List']
        assert obj[0].Name == zonename + ' Air Inlet Node List'
        assert obj[0].Node_1_Name == zonename + ' VRF Indoor Unit Supply Outlet'
    
        obj = [i
               for i
               in idf1.idfobjects['NodeList']
               if i.Name == zonename+' Air Exhaust Node List']
        assert obj[0].Name == zonename + ' Air Exhaust Node List'
        assert obj[0].Node_1_Name == zonename + ' VRF Indoor Unit Return'

    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['ZoneHVAC:EquipmentConnections']
               if i.Zone_Name == zonename]
        assert obj[0].Zone_Name == zonename
        assert obj[0].Zone_Conditioning_Equipment_List_Name == zonename + ' Equipment'
        assert obj[0].Zone_Air_Inlet_Node_or_NodeList_Name == zonename + ' Air Inlet Node List'
        assert obj[0].Zone_Air_Exhaust_Node_or_NodeList_Name == zonename + ' Air Exhaust Node List'
        assert obj[0].Zone_Air_Node_Name == zonename + ' Zone Air Node'
        assert obj[0].Zone_Return_Air_Node_or_NodeList_Name == zonename + ' Return Outlet'
        
    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['ZoneHVAC:EquipmentList']
               if i.Name == zonename + ' Equipment']
        # assert obj[0].defaultvalues == False
        assert obj[0].Name == zonename + ' Equipment'
        assert obj[0].Load_Distribution_Scheme == 'SequentialLoad'
        assert obj[0].Zone_Equipment_1_Object_Type == 'ZoneHVAC:TerminalUnit:VariableRefrigerantFlow'
        assert obj[0].Zone_Equipment_1_Name == zonename + ' VRF Indoor Unit'
        assert obj[0].Zone_Equipment_1_Cooling_Sequence == 1
        assert obj[0].Zone_Equipment_1_Heating_or_NoLoad_Sequence == 1
        assert obj[0].Zone_Equipment_1_Sequential_Cooling_Fraction == ''
        assert obj[0].Zone_Equipment_1_Sequential_Heating_Fraction == ''

    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['ZoneHVAC:TerminalUnit:VariableRefrigerantFlow']
               if i.Zone_Terminal_Unit_Name == zonename + ' VRF Indoor Unit']
        assert obj[0].Zone_Terminal_Unit_Name == zonename + ' VRF Indoor Unit'
        assert obj[0].Terminal_Unit_Availability_Schedule == 'On 24/7'
        assert obj[0].Terminal_Unit_Air_Inlet_Node_Name == zonename + ' VRF Indoor Unit Return'
        assert obj[0].Terminal_Unit_Air_Outlet_Node_Name == zonename + ' VRF Indoor Unit Supply Outlet'
        assert obj[0].Cooling_Supply_Air_Flow_Rate == 'autosize'
        assert obj[0].No_Cooling_Supply_Air_Flow_Rate == 'autosize'
        assert obj[0].Heating_Supply_Air_Flow_Rate == 'autosize'
        assert obj[0].No_Heating_Supply_Air_Flow_Rate == 'autosize'
        assert obj[0].Cooling_Outdoor_Air_Flow_Rate == 0
        assert obj[0].Heating_Outdoor_Air_Flow_Rate == 0
        assert obj[0].No_Load_Outdoor_Air_Flow_Rate == 0
        assert obj[0].Supply_Air_Fan_Operating_Mode_Schedule_Name == 'On 24/7'
        assert obj[0].Supply_Air_Fan_Placement == 'DrawThrough'
        assert obj[0].Supply_Air_Fan_Object_Type == 'Fan:ConstantVolume'
        assert obj[0].Supply_Air_Fan_Object_Name == zonename + ' VRF Indoor Unit Supply Fan'
        assert obj[0].Outside_Air_Mixer_Object_Type == ''
        assert obj[0].Outside_Air_Mixer_Object_Name == ''
        assert obj[0].Cooling_Coil_Object_Type == 'Coil:Cooling:DX:VariableRefrigerantFlow'
        assert obj[0].Cooling_Coil_Object_Name == zonename + ' VRF Indoor Unit DX Cooling Coil'
        assert obj[0].Heating_Coil_Object_Type == 'Coil:Heating:DX:VariableRefrigerantFlow'
        assert obj[0].Heating_Coil_Object_Name == zonename + ' VRF Indoor Unit DX Heating Coil'
        assert obj[0].Zone_Terminal_Unit_On_Parasitic_Electric_Energy_Use == 30
        assert obj[0].Zone_Terminal_Unit_Off_Parasitic_Electric_Energy_Use == 20
        assert obj[0].Rated_Heating_Capacity_Sizing_Ratio == ''
        assert obj[0].Availability_Manager_List_Name == ''

    for zonename in zonenames_orig:
        obj = [i
               for i
               in idf1.idfobjects['Coil:Cooling:DX:VariableRefrigerantFlow']
               if i.Name == zonename + ' VRF Indoor Unit DX Cooling Coil']
        assert obj[0].Name == zonename + ' VRF Indoor Unit DX Cooling Coil'
        assert obj[0].Availability_Schedule_Name == 'On 24/7'
        assert obj[0].Gross_Rated_Total_Cooling_Capacity == 'autosize'
        assert obj[0].Gross_Rated_Sensible_Heat_Ratio == 'autosize'
        assert obj[0].Rated_Air_Flow_Rate == 'autosize'
        assert obj[0].Cooling_Capacity_Ratio_Modifier_Function_of_Temperature_Curve_Name == 'VRFTUCoolCapFT'
        assert obj[0].Cooling_Capacity_Modifier_Curve_Function_of_Flow_Fraction_Name == 'VRFACCoolCapFFF'
        assert obj[0].Coil_Air_Inlet_Node == zonename + ' VRF Indoor Unit Return'
        assert obj[0].Coil_Air_Outlet_Node == zonename + ' VRF Indoor Unit DX Cooling Coil Outlet'
        assert obj[0].Name_of_Water_Storage_Tank_for_Condensate_Collection == ''

    for zonename in zonenames_orig:                                       
        obj = [i                                                          
               for i                                                      
               in idf1.idfobjects['Coil:Heating:DX:VariableRefrigerantFlow']               
               if i.Name == zonename + ' VRF Indoor Unit DX Heating Coil']
        assert obj[0].Name == zonename + ' VRF Indoor Unit DX Heating Coil'
        assert obj[0].Availability_Schedule == 'On 24/7'
        assert obj[0].Gross_Rated_Heating_Capacity == 'autosize'
        assert obj[0].Rated_Air_Flow_Rate == 'autosize'
        assert obj[0].Coil_Air_Inlet_Node == zonename + ' VRF Indoor Unit DX Cooling Coil Outlet'
        assert obj[0].Coil_Air_Outlet_Node == zonename + ' VRF Indoor Unit DX Heating Coil Outlet'
        assert obj[0].Heating_Capacity_Ratio_Modifier_Function_of_Temperature_Curve_Name == 'VRFTUHeatCapFT'
        assert obj[0].Heating_Capacity_Modifier_Function_of_Flow_Fraction_Curve_Name == 'VRFACCoolCapFFF'
        
    for zonename in zonenames_orig:                                       
        obj = [i                                                          
               for i                                                      
               in idf1.idfobjects['Fan:ConstantVolume']               
               if i.Name == zonename + ' VRF Indoor Unit Supply Fan']
        assert obj[0].Name == zonename + ' VRF Indoor Unit Supply Fan'
        assert obj[0].Availability_Schedule_Name == 'On 24/7'
        assert obj[0].Fan_Total_Efficiency == 0.7
        assert obj[0].Pressure_Rise == 100
        assert obj[0].Maximum_Flow_Rate == 'autosize'
        assert obj[0].Motor_Efficiency == 0.9
        assert obj[0].Motor_In_Airstream_Fraction == 1
        assert obj[0].Air_Inlet_Node_Name == zonename + ' VRF Indoor Unit DX Heating Coil Outlet'
        assert obj[0].Air_Outlet_Node_Name == zonename + ' VRF Indoor Unit Supply Outlet'
        assert obj[0].EndUse_Subcategory == 'General'



"""Add EnergyPlus objects in common to both SingleZone and MultipleZone."""


def setComfFieldsPeople(self, verboseMode: bool = True):
    """
    Amend PEOPLE objects so that accim can work.

    Copy existing PEOPLE objects and adds AdaptiveASH55 and AdaptiveCEN15251
    to Thermal Comfort Model types 1 and 2 fields of the existing People
    objects.
    """
    peoplelist = ([people for people in self.idf1.idfobjects['PEOPLE']])
    for i in range(len(peoplelist)):
        self.idf1.newidfobject(
            'PEOPLE',
            Name=peoplelist[i].Name,
            Zone_or_ZoneList_Name=peoplelist[i].Zone_or_ZoneList_Name,
            Number_of_People_Schedule_Name=peoplelist[i].Number_of_People_Schedule_Name,
            Number_of_People_Calculation_Method=peoplelist[i].Number_of_People_Calculation_Method,
            Number_of_People=peoplelist[i].Number_of_People,
            People_per_Zone_Floor_Area=peoplelist[i].People_per_Zone_Floor_Area,
            Zone_Floor_Area_per_Person=peoplelist[i].Zone_Floor_Area_per_Person,
            Fraction_Radiant=peoplelist[i].Fraction_Radiant,
            Sensible_Heat_Fraction=peoplelist[i].Sensible_Heat_Fraction,
            Activity_Level_Schedule_Name=peoplelist[i].Activity_Level_Schedule_Name,
            Carbon_Dioxide_Generation_Rate=peoplelist[i].Carbon_Dioxide_Generation_Rate,
            Enable_ASHRAE_55_Comfort_Warnings=peoplelist[i].Enable_ASHRAE_55_Comfort_Warnings,
            Mean_Radiant_Temperature_Calculation_Type=peoplelist[i].Mean_Radiant_Temperature_Calculation_Type,
            Surface_NameAngle_Factor_List_Name=peoplelist[i].Surface_NameAngle_Factor_List_Name,
            Work_Efficiency_Schedule_Name=peoplelist[i].Work_Efficiency_Schedule_Name,
            Clothing_Insulation_Calculation_Method=peoplelist[i].Clothing_Insulation_Calculation_Method,
            Clothing_Insulation_Calculation_Method_Schedule_Name=peoplelist[i].Clothing_Insulation_Calculation_Method_Schedule_Name,
            Clothing_Insulation_Schedule_Name=peoplelist[i].Clothing_Insulation_Schedule_Name,
            Air_Velocity_Schedule_Name=peoplelist[i].Air_Velocity_Schedule_Name,
            Thermal_Comfort_Model_1_Type='AdaptiveASH55',
            Thermal_Comfort_Model_2_Type='AdaptiveCEN15251',
            Thermal_Comfort_Model_3_Type='',
            Thermal_Comfort_Model_4_Type='',
            Thermal_Comfort_Model_5_Type='',
            )
        firstpeopleobject = self.idf1.idfobjects['PEOPLE'][0]
        self.idf1.removeidfobject(firstpeopleobject)
    peoplelist = ([people for people in self.idf1.idfobjects['PEOPLE']])
    if verboseMode:
        print('The people objects in the model are:')
        print(*peoplelist,sep="\n")
    del peoplelist, firstpeopleobject


def addOpTempTherm(self, verboseMode : bool = True):
    """
    Amend ZoneControl:Thermostat:OperativeTemperature objects.

    Add ZoneControl:Thermostat:OperativeTemperature objects for each zone.
    """
    for zonename_orig in self.zonenames_orig:
        if zonename_orig+' Thermostat' in [thermostat.Thermostat_Name for thermostat in self.idf1.idfobjects['ZoneControl:Thermostat:OperativeTemperature']]:
            if verboseMode:
                print(zonename_orig+' Thermostat already was in the model')
        else:
            self.idf1.newidfobject(
                'ZoneControl:Thermostat:OperativeTemperature',
                Thermostat_Name=zonename_orig+" Thermostat",
                Radiative_Fraction_Input_Mode="Scheduled",
                Fixed_Radiative_Fraction='',
                Radiative_Fraction_Schedule_Name='TypOperativeTempControlSch'
                )
            if verboseMode:
                print(zonename_orig+' Thermostat has been added')


def addBaseSchedules(self, verboseMode : bool = True):
    """
    Amend Schedule:Compact objects.

    Checks Schedule:Compact objects needed for SingleZone accim,
    and add them in case these are not in the model
    """
    if "On" in [schedule.Name for schedule in self.idf1.idfobjects['Schedule:Compact']]:
        if verboseMode:
            print("On Schedule already was in the model")
    else:
        self.idf1.newidfobject(
            'Schedule:Compact',
            Name="On",
            Schedule_Type_Limits_Name="Any Number",
            Field_1='Through: 12/31',
            Field_2='For: AllDays',
            Field_3='Until: 24:00,1'
            )
        if verboseMode:
            print("On Schedule has been added")


def setAvailSchOn(self, verboseMode: bool = True):
    """
    Amend availability schedules.

    Assign On Compact:Schedule to heating and cooling availability
    schedule names.
    """
    for schedule in [i for i in self.idf1.idfobjects['ZoneHVAC:IdealLoadsAirSystem']]:
        schedule.Heating_Availability_Schedule_Name='On'
        schedule.Cooling_Availability_Schedule_Name='On'
    if verboseMode:
        print('All ZoneHVAC:IdealLoadsAirSystem Heating and Cooling availability schedules has been set to on')

def saveaccim(self, verboseMode: bool = True):
    """Save IDF."""
    self.idf1.save()
    if verboseMode:
        print('IDF has been saved')




# -*- coding: utf-8 -*-
"""
Created on Sun Feb 28 17:03:48 2021

@author: Daniel
"""


from accim.sim import accim_Main

def test_addEMSProgramsBase():
    
    from eppy.modeleditor import IDF
    
    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)
    
    idf0 = IDF('TestModel_SingleZone.idf')
    
    programlist = ([program.Name for program in idf0.idfobjects['EnergyManagementSystem:Program']])
    assert ('SetComfTemp' in programlist) == False
    
    z = accim_Main.accimobj_SingleZone_Ep94('TestModel_SingleZone')
    z.addEMSProgramsBase()
    z.saveaccim()
    
    idf1 = IDF('TestModel_SingleZone_pymod.idf')
    
    programlist = ([program.Name for program in idf1.idfobjects['EnergyManagementSystem:Program']])
    assert ('SetComfTemp' in programlist) == True

    SetComfTemp = ([program
                    for program
                    in idf1.idfobjects['EnergyManagementSystem:Program']
                    if program.Name == 'SetComfTemp'])

    assert SetComfTemp[0].Program_Line_1 == 'if AdapStand == 1'
    assert SetComfTemp[0].Program_Line_2 == 'set ComfTemp = RMOT*0.33+18.8'
    assert SetComfTemp[0].Program_Line_3 == 'elseif AdapStand == 2'
    assert SetComfTemp[0].Program_Line_4 == 'set ComfTemp = PMOT*0.31+17.8'
    assert SetComfTemp[0].Program_Line_5 == 'endif'

    print(SetComfTemp[0].Name)
    print(SetComfTemp[0].Program_Line_1)
    
# test_addEMSProgramsBase()



import setuptools
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
      name='accim',
      version='0.1.4',
      description="Transforms PMV-based into adaptive setpoint temperature EnergyPlus building energy models",
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/dsanchez-garcia/accim',
      author='Daniel Sánchez-García',
      author_email='dsanchez7@us.es',
      license="MIT License",
      classifiers=[
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3',
          "Intended Audience :: Science/Research",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Topic :: Scientific/Engineering"
          ],
      packages=setuptools.find_packages(),
      package_data={
           "": ["*.csv", "*.idf"]
           },
      install_requires=[
           "eppy"
           ],
      scripts=['bin/accis.py'],
      keywords=[
          'adaptive thermal comfort',
          'building energy model',
          'building performance simulation',
          'energy efficiency'
          ]
      )



# -*- coding: utf-8 -*-
"""
Created on Sun Feb 28 17:03:48 2021

@author: Daniel
"""

from eppy import modeleditor
from eppy.modeleditor import IDF

import pytest

from accim.sim import accim_Base_EMS


def lookfor(self, name, objectType):
    """
    Look for name in objectType list.
    """
    listToSearchIn = ([x.Name for x in self.idf1.idfobjects[objectType]])

    if name in listToSearchIn:
        return True
    else:
        return False


def test_addEMSProgramsBase(self):
    pass
    



def test_accimobj_SingleZone_Ep94():
    from accim.sim import accim_Main
    import os
    originalname = 'TestModel_SingleZone'
    newidfname = originalname + '_pymod'

    z = accim_Main.accimobj_SingleZone_Ep94(
        filename_temp=originalname,
        verboseMode=False
        )

    assert (newidfname + '.idf') in [i for i in os.listdir() if '_pymod' in i]
    os.remove(newidfname+'.idf')

def test_accimobj_MultipleZone_Ep94():
    from accim.sim import accim_Main
    import os
    originalname = 'TestModel_MultipleZone'
    newidfname = originalname + '_pymod'

    z = accim_Main.accimobj_MultipleZone_Ep94(
        filename_temp=originalname,
        verboseMode=False
        )

    assert (newidfname + '.idf') in [i for i in os.listdir() if '_pymod' in i]
    os.remove(newidfname+'.idf')


def test_accimInstance():
    from accim.sim import accim_Main
    import os
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    originalname = 'TestModel_SingleZone'
    newidfname = originalname + '_pymod'
    idf0 = IDF(originalname + '.idf')

    zonenames_orig = ([zone.Name for zone in idf0.idfobjects['ZONE']])
    z = accim_Main.accimInstance(
        filename_temp=originalname,
        ScriptType='sz',
        EnergyPlus_version='ep94',
        verboseMode=False
    )
    assert (newidfname + '.idf') in [i for i in os.listdir() if '_pymod' in i]
    assert zonenames_orig == z.zonenames_orig
    os.remove(newidfname + '.idf')

    originalname = 'TestModel_MultipleZone'
    newidfname = originalname + '_pymod'
    idf0 = IDF(originalname + '.idf')

    zonenames_orig = ([zone.Name for zone in idf0.idfobjects['ZONE']])
    windownamelist_orig = (
        [window.Name for window in idf0.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if
         window.Name.endswith('_Win')])
    z = accim_Main.accimInstance(
        filename_temp=originalname,
        ScriptType='mz',
        EnergyPlus_version='ep94',
        verboseMode=False
    )

    assert (newidfname + '.idf') in [i for i in os.listdir() if '_pymod' in i]
    assert zonenames_orig == z.zonenames_orig
    assert windownamelist_orig == z.windownamelist_orig
    os.remove(newidfname + '.idf')



import pytest
from accim.sim import accim_Main

@pytest.fixture()
def IDFobject():
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimInstance(
        filename_temp='TestModel_SingleZone',
        ScriptType='sz',
        EnergyPlus_version='ep94',
        verboseMode=False)
    return z

def test_addForscriptSchSingleZone(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addForscriptSchSingleZone(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    obj = ([x
            for x
            in idf1.idfobjects['Schedule:Compact']
            if x.Name == "FORSCRIPT_AHST"])
    assert obj[0].Name == "FORSCRIPT_AHST"
    assert obj[0].Schedule_Type_Limits_Name == "Any Number"
    assert obj[0].Field_1 == 'Through: 12/31'
    assert obj[0].Field_2 == 'For: AllDays'
    assert obj[0].Field_3 == 'Until: 24:00'
    assert obj[0].Field_4 == '20'

    obj = ([x
            for x
            in idf1.idfobjects['Schedule:Compact']
            if x.Name == "FORSCRIPT_ACST"])
    assert obj[0].Name == "FORSCRIPT_ACST"
    assert obj[0].Schedule_Type_Limits_Name == "Any Number"
    assert obj[0].Field_1 == 'Through: 12/31'
    assert obj[0].Field_2 == 'For: AllDays'
    assert obj[0].Field_3 == 'Until: 24:00'
    assert obj[0].Field_4 == '24'

    for x in [x for x in idf1.idfobjects['ThermostatSetpoint:DualSetpoint']]:
        assert x.Heating_Setpoint_Temperature_Schedule_Name == "FORSCRIPT_AHST"
        assert x.Cooling_Setpoint_Temperature_Schedule_Name == "FORSCRIPT_ACST"



"""Classes for accim SingleZone and MultipleZone."""


class accimobj():
    """accim Object."""

    from os import listdir
    import numpy

    from accim.sim.accim_IDFgeneration import inputdataSingleZone
    from accim.sim.accim_IDFgeneration import inputdataMultipleZone
    from accim.sim.accim_IDFgeneration import genIDFSingleZone
    from accim.sim.accim_IDFgeneration import genIDFMultipleZone

    from accim.sim.accim_Base import setComfFieldsPeople
    from accim.sim.accim_Base import addOpTempTherm
    from accim.sim.accim_Base import addBaseSchedules
    from accim.sim.accim_Base import setAvailSchOn
    from accim.sim.accim_Base import saveaccim

    from accim.sim.accim_Base_EMS import addEMSProgramsBase
    from accim.sim.accim_Base_EMS import addEMSPCMBase
    from accim.sim.accim_Base_EMS import addEMSOutputVariableBase
    from accim.sim.accim_Base_EMS import addOutputVariablesTimestep
    from accim.sim.accim_Base_EMS import addSimplifiedOutputVariables

    pass


class accimobj_SingleZone_Ep91 (accimobj):
    """SingleZone accim object."""

    from accim.sim.accim_SingleZone import addForscriptSchSingleZone

    from accim.sim.accim_SingleZone_EMS import addGlobVarListSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSSensorsSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSActuatorsSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSProgramsSingleZone
    from accim.sim.accim_SingleZone_EMS import addOutputVariablesSingleZone

    def __init__(self, filename_temp, verboseMode: bool = True):
        from eppy import modeleditor
        from eppy.modeleditor import IDF

        iddfile = 'C:/EnergyPlusV9-1-0/Energy+.idd'
        IDF.setiddname(iddfile)

        fname1 = filename_temp+'.idf'
        self.idf0 = IDF(fname1)
        self.idf0.savecopy(filename_temp+'_pymod.idf')

        self.filename = filename_temp+'_pymod'
        fname1 = self.filename+'.idf'
        self.idf1 = IDF(fname1)
        self.filename = filename_temp+'_pymod'

        # print(self.filename)

        self.zonenames_orig = ([zone.Name for zone in self.idf1.idfobjects['ZONE']])
        # print(self.zonenames_orig)

        self.zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
        # print(self.zonenames)
        if verboseMode:
            print(f'The zones in the model {filename_temp} are:')
            print(*self.zonenames, sep="\n")

class accimobj_MultipleZone_Ep91 (accimobj):
    """MultipleZone accim object."""

    from accim.sim.accim_MultipleZone import addMultipleZoneSch
    from accim.sim.accim_MultipleZone import addCurveObj
    from accim.sim.accim_MultipleZone import addDetHVACobjEp91
    from accim.sim.accim_MultipleZone import addForscriptSchMultipleZone
    from accim.sim.accim_MultipleZone import checkVentIsOn

    from accim.sim.accim_MultipleZone_EMS import addGlobVarListMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSSensorsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSActuatorsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSProgramsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSOutputVariableMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addOutputVariablesMultipleZone

    def __init__(self, filename_temp, verboseMode: bool = True):
        from eppy import modeleditor
        from eppy.modeleditor import IDF

        iddfile = 'C:/EnergyPlusV9-1-0/Energy+.idd'
        IDF.setiddname(iddfile)

        fname1 = filename_temp+'.idf'
        self.idf0 = IDF(fname1)
        self.idf0.savecopy(filename_temp+'_pymod.idf')

        self.filename = filename_temp+'_pymod'
        fname1 = self.filename+'.idf'
        self.idf1 = IDF(fname1)
        self.filename = filename_temp+'_pymod'

        # print(self.filename)

        self.zonenames_orig = ([zone.Name for zone in self.idf1.idfobjects['ZONE']])
        # print(self.zonenames_orig)

        self.zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
        # print(self.zonenames)

        self.windownamelist_orig = ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])
        # print(self.windownamelist_orig)
        self.windownamelist_orig_split = ([i.split('_') for i in self.windownamelist_orig])
        # print(self.windownamelist_orig_split)

        self.windownamelist = ([sub.replace(':', '_') for sub in ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])])
        # print(self.windownamelist)
        if verboseMode:
            print(f'The zones in the model {filename_temp} are:')
            print(*self.zonenames, sep="\n")
            print(f'The windows in the model {filename_temp} are:')
            print(*self.windownamelist, sep="\n")

class accimobj_SingleZone_Ep94 (accimobj):
    """SingleZone accim object."""

    from accim.sim.accim_SingleZone import addForscriptSchSingleZone

    from accim.sim.accim_SingleZone_EMS import addGlobVarListSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSSensorsSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSActuatorsSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSProgramsSingleZone
    from accim.sim.accim_SingleZone_EMS import addOutputVariablesSingleZone

    def __init__(self, filename_temp, verboseMode: bool = True):
        from eppy import modeleditor
        from eppy.modeleditor import IDF

        iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
        IDF.setiddname(iddfile)

        fname1 = filename_temp+'.idf'
        self.idf0 = IDF(fname1)
        self.idf0.savecopy(filename_temp+'_pymod.idf')

        self.filename = filename_temp+'_pymod'
        fname1 = self.filename+'.idf'
        self.idf1 = IDF(fname1)
        self.filename = filename_temp+'_pymod'

        # print(self.filename)

        self.zonenames_orig = ([zone.Name for zone in self.idf1.idfobjects['ZONE']])
        # print(self.zonenames_orig)

        self.zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
        # print(self.zonenames)
        if verboseMode:
            print(f'The zones in the model {filename_temp} are:')
            print(*self.zonenames, sep="\n")


class accimobj_MultipleZone_Ep94 (accimobj):
    """MultipleZone accim object."""

    from accim.sim.accim_MultipleZone import addMultipleZoneSch
    from accim.sim.accim_MultipleZone import addCurveObj
    from accim.sim.accim_MultipleZone import addDetHVACobjEp94
    from accim.sim.accim_MultipleZone import addForscriptSchMultipleZone
    from accim.sim.accim_MultipleZone import checkVentIsOn

    from accim.sim.accim_MultipleZone_EMS import addGlobVarListMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSSensorsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSActuatorsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSProgramsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSOutputVariableMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addOutputVariablesMultipleZone

    def __init__(self, filename_temp, verboseMode: bool = True):
        from eppy import modeleditor
        from eppy.modeleditor import IDF

        iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
        IDF.setiddname(iddfile)

        fname1 = filename_temp+'.idf'
        self.idf0 = IDF(fname1)
        self.idf0.savecopy(filename_temp+'_pymod.idf')

        self.filename = filename_temp+'_pymod'
        fname1 = self.filename+'.idf'
        self.idf1 = IDF(fname1)
        self.filename = filename_temp+'_pymod'

        # print(self.filename)

        self.zonenames_orig = ([zone.Name for zone in self.idf1.idfobjects['ZONE']])
        # print(self.zonenames_orig)

        self.zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
        # print(self.zonenames)

        self.windownamelist_orig = ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])
        # print(self.windownamelist_orig)
        self.windownamelist_orig_split = ([i.split('_') for i in self.windownamelist_orig])
        # print(self.windownamelist_orig_split)

        self.windownamelist = ([sub.replace(':', '_') for sub in ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])])
        # print(self.windownamelist)
        if verboseMode:
            print(f'The zones in the model {filename_temp} are:')
            print(*self.zonenames, sep="\n")
            print(f'The windows in the model {filename_temp} are:')
            print(*self.windownamelist, sep="\n")




class accimInstance():
    """MultipleZone accim object."""

    from os import listdir
    import numpy

    from accim.sim.accim_IDFgeneration import inputdataSingleZone
    from accim.sim.accim_IDFgeneration import inputdataMultipleZone
    from accim.sim.accim_IDFgeneration import genIDFSingleZone
    from accim.sim.accim_IDFgeneration import genIDFMultipleZone

    from accim.sim.accim_Base import setComfFieldsPeople
    from accim.sim.accim_Base import addOpTempTherm
    from accim.sim.accim_Base import addBaseSchedules
    from accim.sim.accim_Base import setAvailSchOn
    from accim.sim.accim_Base import saveaccim

    from accim.sim.accim_Base_EMS import addEMSProgramsBase
    from accim.sim.accim_Base_EMS import addEMSPCMBase
    from accim.sim.accim_Base_EMS import addEMSOutputVariableBase
    from accim.sim.accim_Base_EMS import addOutputVariablesTimestep
    from accim.sim.accim_Base_EMS import addSimplifiedOutputVariables


    from accim.sim.accim_SingleZone import addForscriptSchSingleZone

    from accim.sim.accim_SingleZone_EMS import addGlobVarListSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSSensorsSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSActuatorsSingleZone
    from accim.sim.accim_SingleZone_EMS import addEMSProgramsSingleZone
    from accim.sim.accim_SingleZone_EMS import addOutputVariablesSingleZone


    from accim.sim.accim_MultipleZone import addMultipleZoneSch
    from accim.sim.accim_MultipleZone import addCurveObj
    from accim.sim.accim_MultipleZone import addDetHVACobjEp91
    from accim.sim.accim_MultipleZone import addDetHVACobjEp94
    from accim.sim.accim_MultipleZone import addForscriptSchMultipleZone
    from accim.sim.accim_MultipleZone import checkVentIsOn

    from accim.sim.accim_MultipleZone_EMS import addGlobVarListMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSSensorsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSActuatorsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSProgramsMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addEMSOutputVariableMultipleZone
    from accim.sim.accim_MultipleZone_EMS import addOutputVariablesMultipleZone


    def __init__(self, filename_temp, ScriptType: str = None, EnergyPlus_version: str = None, verboseMode: bool = True):
        from eppy import modeleditor
        from eppy.modeleditor import IDF
        if EnergyPlus_version.lower() == 'ep94':
            iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
        elif EnergyPlus_version.lower() == 'ep91':
            iddfile = 'C:/EnergyPlusV9-1-0/Energy+.idd'
        else:
            raise ValueError("""EnergyPlus version not supported.\n
                                     Only works for EnergyPlus 9.1 (enter Ep91) and
                                     EnergyPlus 9.4 (enter Ep94) versions.""")
        IDF.setiddname(iddfile)

        fname1 = filename_temp+'.idf'
        self.idf0 = IDF(fname1)
        self.idf0.savecopy(filename_temp+'_pymod.idf')

        self.filename = filename_temp+'_pymod'
        fname1 = self.filename+'.idf'
        self.idf1 = IDF(fname1)
        self.filename = filename_temp+'_pymod'

        # print(self.filename)

        self.zonenames_orig = ([zone.Name for zone in self.idf1.idfobjects['ZONE']])
        # print(self.zonenames_orig)

        self.zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
        # print(self.zonenames)
        if verboseMode:
            print(f'The zones in the model {filename_temp} are:')
            print(*self.zonenames, sep="\n")

        if ScriptType.lower() == 'mz' or ScriptType.lower() == 'multiplezone':
            self.windownamelist_orig = ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])
            # print(self.windownamelist_orig)
            self.windownamelist_orig_split = ([i.split('_') for i in self.windownamelist_orig])
            # print(self.windownamelist_orig_split)

            self.windownamelist = ([sub.replace(':', '_') for sub in ([window.Name for window in self.idf1.idfobjects['AirflowNetwork:MultiZone:Component:DetailedOpening'] if window.Name.endswith('_Win')])])
            # print(self.windownamelist)
            if verboseMode:
                print(f'The windows in the model {filename_temp} are:')
                print(*self.windownamelist, sep="\n")
